@@ -5,9 +5,7 @@ and strains and plotting the Knapmeyer et al. (2006) tectonic dataset.
 
 import numpy as np
 import pyshtools as pysh
-import matplotlib.pyplot as plt
 from pathlib import Path
-import time
 
 pi = np.pi
 
@@ -16,8 +14,8 @@ pi = np.pi
 
 def SH_deriv(theta, phi, lmax):
     """
-    Compute on the fly spherical harmonic derivatives
-    (first and second order)
+    Compute spherical harmonic derivatives at a given
+    location (first and second order).
 
     Returns
     -------
@@ -37,7 +35,8 @@ def SH_deriv(theta, phi, lmax):
         Array with the first derivative
         of Legendre polynomials with respect to colatitude and longitude.
     y_lm : array, size(2,lmax+1,lmax+1)
-        Array of spherical harmonic functions
+        Array of spherical harmonic functions.
+
     Parameters
     ----------
     theta : float
@@ -47,25 +46,28 @@ def SH_deriv(theta, phi, lmax):
     lmax : int
         Maximum spherical harmonic degree to compute for the derivatives.
     """
-    Y_lm_d1_theta_a = np.zeros((2, lmax + 1, lmax + 1))
-    Y_lm_d1_phi_a = np.zeros((2, lmax + 1, lmax + 1))
-    Y_lm_d2_phi_a = np.zeros((2, lmax + 1, lmax + 1))
-    Y_lm_d2_thetaphi_a = np.zeros((2, lmax + 1, lmax + 1))
-    Y_lm_d2_theta_a = np.zeros((2, lmax + 1, lmax + 1))
-    y_lm = np.zeros((2, lmax + 1, lmax + 1))
+    shape = (2, lmax + 1, lmax + 1)
+    Y_lm_d1_theta_a = np.zeros(shape)
+    Y_lm_d1_phi_a = np.zeros(shape)
+    Y_lm_d2_phi_a = np.zeros(shape)
+    Y_lm_d2_thetaphi_a = np.zeros(shape)
+    Y_lm_d2_theta_a = np.zeros(shape)
+    y_lm = np.zeros(shape)
 
+    cost = np.cos(theta)
+    sint = np.sin(theta)
     if theta == 0 or theta == pi:
         dp_theta = np.zeros((int((lmax + 1) * (lmax + 2) / 2)))
         p_theta = np.zeros((int((lmax + 1) * (lmax + 2) / 2)))
         costsint = 0.0
         sintt = 0.0
     else:
-        p_theta, dp_theta = pysh.legendre.PlmBar_d1(lmax, np.cos(theta))
-        dp_theta *= -np.sin(theta)  # Derivative with respect to
+        p_theta, dp_theta = pysh.legendre.PlmBar_d1(lmax, cost)
+        dp_theta *= -sint  # Derivative with respect to
         # theta.
-        p_theta /= np.sin(theta)
-        costsint = np.cos(theta) / np.sin(theta)
-        sintt = 1.0 / np.sin(theta) ** 2
+        p_theta /= sint
+        costsint = cost / sint
+        sintt = 1.0 / sint ** 2
     for l in range(lmax + 1):
         lapla = float(-l * (l + 1))
         for m in range(-l, l + 1):
@@ -82,7 +84,7 @@ def SH_deriv(theta, phi, lmax):
                 Y_lm_d1_phi_a[0, l, m] = p_theta[index] * msinmphi
                 Y_lm_d2_phi_a[0, l, m] = p_theta[index] * m2cosphi
                 Y_lm_d2_thetaphi_a[0, l, m] = dp_theta[index] * msinmphi
-                y_lm[0, l, m] = p_theta[index] * np.sin(theta) * cosmphi
+                y_lm[0, l, m] = p_theta[index] * cosmphi
             else:
                 mcosmphi = m_abs * cosmphi
                 m2sinphi = -(m_abs ** 2) * sinmphi
@@ -90,8 +92,9 @@ def SH_deriv(theta, phi, lmax):
                 Y_lm_d1_phi_a[1, l, m_abs] = p_theta[index] * mcosmphi
                 Y_lm_d2_phi_a[1, l, m_abs] = p_theta[index] * m2sinphi
                 Y_lm_d2_thetaphi_a[1, l, m_abs] = dp_theta[index] * mcosmphi
-                y_lm[1, l, m_abs] = p_theta[index] * np.sin(theta) * sinmphi
+                y_lm[1, l, m_abs] = p_theta[index] * sinmphi
 
+        y_lm[:, l, : l + 1] *= sint
         if theta == 0 or theta == pi:
             Y_lm_d2_theta_a[:, l, : l + 1] = 0.0  # Not defined.
         else:
@@ -116,10 +119,10 @@ def SH_deriv(theta, phi, lmax):
 # ==== SH_deriv_store ====
 
 
-def SH_deriv_store(lmax, path, save=True):
+def SH_deriv_store(lmax, path, save=True, compressed=False):
     """
     Compute and store or load spherical harmonic derivatives
-    (first and second order).
+    over the entire sphere (first and second order).
 
     Returns
     -------
@@ -138,6 +141,8 @@ def SH_deriv_store(lmax, path, save=True):
     Y_lm_d2_thetaphi_a : array, size(2,lmax+1,lmax+1)
         Array with the first derivative
         of Legendre polynomials with respect to colatitude and longitude.
+    y_lm_save : array, size(2,lmax+1,lmax+1)
+        Array of spherical harmonic functions.
 
     Parameters
     ----------
@@ -145,11 +150,17 @@ def SH_deriv_store(lmax, path, save=True):
         Path to store or load spherical harmonic derivatives.
     lmax : int
         Maximum spherical harmonic degree to compute for the derivatives.
-    save : boolean
-        Whether the data is saved at the given path location.
+        This parameter determines the resolution of the grid as
+        latitude(2 * lmax + 2) and longitude(2 * (2 * lmax + 2))
+    save : boolean, optional, default = True
+        If True, save the data at the given path location.
+    compressed : boolean, optional, default = False
+        If True, the data is saved in compressed .npz format instead of
+        npy, which decreases the file size by about a factor 2. This is
+        recommended when lmax > 75.
     """
     n = 2 * lmax + 2
-    poly_file = "%s/Y_lmsd1d2_lmax%s.npy" % (path, lmax)
+    poly_file = "%s/Y_lmsd1d2_lmax%s.%s" % (path, lmax, "npz" if compressed else "npy")
 
     if Path(poly_file).exists() == 0:
         print(
@@ -157,11 +168,13 @@ def SH_deriv_store(lmax, path, save=True):
             + " time depending on lmax, input lmax is %s" % (lmax)
         )
         index_size = int((lmax + 1) * (lmax + 2) / 2)
-        Y_lm_d1_theta_a = np.zeros((n, 2 * n, 2, lmax + 1, lmax + 1))
-        Y_lm_d1_phi_a = np.zeros((n, 2 * n, 2, lmax + 1, lmax + 1))
-        Y_lm_d2_phi_a = np.zeros((n, 2 * n, 2, lmax + 1, lmax + 1))
-        Y_lm_d2_thetaphi_a = np.zeros((n, 2 * n, 2, lmax + 1, lmax + 1))
-        Y_lm_d2_theta_a = np.zeros((n, 2 * n, 2, lmax + 1, lmax + 1))
+        shape_save = (n, 2 * n, 2, lmax + 1, lmax + 1)
+        Y_lm_d1_theta_a = np.zeros(shape_save)
+        Y_lm_d1_phi_a = np.zeros(shape_save)
+        Y_lm_d2_phi_a = np.zeros(shape_save)
+        Y_lm_d2_thetaphi_a = np.zeros(shape_save)
+        Y_lm_d2_theta_a = np.zeros(shape_save)
+        y_lm_save = np.zeros(shape_save)
         phi_ar = np.linspace(0, 360, 2 * n, endpoint=False) * pi / 180.0
         y_lm = np.zeros((len(phi_ar), 2, lmax + 1, lmax + 1))
 
@@ -169,16 +182,18 @@ def SH_deriv_store(lmax, path, save=True):
         for theta in np.linspace(0, 180, n, endpoint=False) * pi / 180.0:
             print(" colatitude %s of 180" % (int(theta * 180 / pi)), end="\r")
             t_i += 1
+            sint = np.sin(theta)
+            cost = np.cos(theta)
             if theta == 0 or theta == pi:
                 dp_theta = np.zeros((index_size))
                 p_theta = np.zeros((index_size))
             else:
-                p_theta, dp_theta = pysh.legendre.PlmBar_d1(lmax, np.cos(theta))
-                dp_theta *= -np.sin(theta)  # Derivative with
+                p_theta, dp_theta = pysh.legendre.PlmBar_d1(lmax, cost)
+                dp_theta *= -sint  # Derivative with
                 # respect to theta.
-                p_theta /= np.sin(theta)
-                costsint = np.cos(theta) / np.sin(theta)
-                sintt = 1.0 / np.sin(theta) ** 2
+                p_theta /= sint
+                costsint = cost / sint
+                sintt = 1.0 / sint ** 2
 
             for l in range(lmax + 1):
                 lapla = float(-l * (l + 1))
@@ -210,7 +225,8 @@ def SH_deriv_store(lmax, path, save=True):
                         )
                         y_lm[:, 1, l, m_abs] = p_theta[index] * sinmphi
 
-                y_lm[:, :, l, : l + 1] *= np.sin(theta)
+                y_lm[:, :, l, : l + 1] *= sint
+                y_lm_save[t_i, :, :, l, : l + 1] = y_lm[:, :, l, : l + 1]
 
                 if theta == 0 or theta == pi:
                     Y_lm_d2_theta_a[t_i, :, :, l, : l + 1] = 0.0
@@ -224,26 +240,52 @@ def SH_deriv_store(lmax, path, save=True):
                         - sintt * Y_lm_d2_phi_a[t_i, :, :, l, : l + 1]
                     )
         if save:
-            np.save(
-                poly_file,
-                [
-                    Y_lm_d1_theta_a,
-                    Y_lm_d1_phi_a,
-                    Y_lm_d2_theta_a,
-                    Y_lm_d2_phi_a,
-                    Y_lm_d2_thetaphi_a,
-                ],
-            )
+            if compressed:
+                np.savez_compressed(
+                    poly_file,
+                    Y_lm_d1_t=Y_lm_d1_theta_a,
+                    Y_lm_d1_p=Y_lm_d1_phi_a,
+                    Y_lm_d2_t=Y_lm_d2_theta_a,
+                    Y_lm_d2_p=Y_lm_d2_phi_a,
+                    Y_lm_d2_tp=Y_lm_d2_thetaphi_a,
+                    Y_lm=y_lm_save,
+                )
+            else:
+                np.save(
+                    poly_file,
+                    [
+                        Y_lm_d1_theta_a,
+                        Y_lm_d1_phi_a,
+                        Y_lm_d2_theta_a,
+                        Y_lm_d2_phi_a,
+                        Y_lm_d2_thetaphi_a,
+                        y_lm_save,
+                    ],
+                )
     else:
-        print("Loading precomputed SH derivatives for strain calculations")
-        (
-            Y_lm_d1_theta_a,
-            Y_lm_d1_phi_a,
-            Y_lm_d2_theta_a,
-            Y_lm_d2_phi_a,
-            Y_lm_d2_thetaphi_a,
-        ) = np.load(poly_file, allow_pickle=True)
-        print("Loading done")
+        if compressed:
+            print(
+                "Loading precomputed compressed SH derivatives for strain calculations"
+            )
+            data = np.load(poly_file)
+            Y_lm_d1_theta_a = data["Y_lm_d1_t"]
+            Y_lm_d1_phi_a = data["Y_lm_d1_p"]
+            Y_lm_d2_theta_a = data["Y_lm_d2_t"]
+            Y_lm_d2_phi_a = data["Y_lm_d2_p"]
+            Y_lm_d2_thetaphi_a = data["Y_lm_d2_tp"]
+            y_lm_save = data["Y_lm"]
+            print("Loading done")
+        else:
+            print("Loading precomputed SH derivatives for strain calculations")
+            (
+                Y_lm_d1_theta_a,
+                Y_lm_d1_phi_a,
+                Y_lm_d2_theta_a,
+                Y_lm_d2_phi_a,
+                Y_lm_d2_thetaphi_a,
+                y_lm_save,
+            ) = np.load(poly_file, allow_pickle=True)
+            print("Loading done")
 
     return (
         Y_lm_d1_theta_a,
@@ -251,6 +293,7 @@ def SH_deriv_store(lmax, path, save=True):
         Y_lm_d2_theta_a,
         Y_lm_d2_phi_a,
         Y_lm_d2_thetaphi_a,
+        y_lm_save,
     )
 
 
@@ -270,12 +313,12 @@ def Displacement_strains(
     lon_min=0,
     lon_max=360,
     only_deflec=False,
-    precomp=True,
     Y_lm_d1_t=None,
     Y_lm_d1_p=None,
     Y_lm_d2_t=None,
     Y_lm_d2_p=None,
     Y_lm_d2_tp=None,
+    y_lm=None,
     path=None,
     quiet=True,
 ):
@@ -287,22 +330,22 @@ def Displacement_strains(
     -------
     stress_theta : array, size(2*lmax+2,2*(2*lmax+2))
         Array with the stress field with respect to colatitude.
-        This is equation A12 from Banerdt (1986)
+        This is equation A12 from Banerdt (1986).
     stress_phi : array, size(2,lmax+1,lmax+1)
         Array with the stress field with respect to longitude.
-        This is equation A13 from Banerdt (1986)
+        This is equation A13 from Banerdt (1986).
     stress_theta_phi : array, size(2,lmax+1,lmax+1)
         Array with the stress field with respect to colatitude and longitude.
-        This is equation A14 from Banerdt (1986)
+        This is equation A14 from Banerdt (1986).
     eps_theta : array, size(2,lmax+1,lmax+1)
         Array with the elongation with respect to colatitude.
-        This is equation A16 from Banerdt (1986)
+        This is equation A16 from Banerdt (1986).
     eps_phi : array, size(2,lmax+1,lmax+1)
         Array with the elongation with respect to longitude.
         This is equation A17 from Banerdt (1986).
     omega : array, size(2,lmax+1,lmax+1)
         Array with the shearing deformation.
-        This is equation A18 from Banerdt (1986). Corrected for the prefactor 2
+        This is equation A18 from Banerdt (1986). Corrected for the prefactor 2.
     kappa_theta : array, size(2,lmax+1,lmax+1)
         Array with the bending deformation with respect to colatitude.
         This is equation A19 from Banerdt (1986).
@@ -311,7 +354,7 @@ def Displacement_strains(
         This is equation A20 from Banerdt (1986).
     tau : array, size(2,lmax+1,lmax+1)
         Array with the twisting deformation.
-        This is equation A21 from Banerdt (1986). Corrected for the prefactor 2
+        This is equation A21 from Banerdt (1986). Corrected for the prefactor 2.
 
     Parameters
     ----------
@@ -339,10 +382,8 @@ def Displacement_strains(
         Minimum longitude for grid computation of strains and stresses.
     lon_max : float, optional, default = 360
         Maximum longitude for grid computation of strains and stresses.
-    only_deflec : int, optional, default = False
+    only_deflec : bool, optional, default = False
         Output only the displacement grid for all latitude and longitudes.
-    precomp : int, optional, default = True
-        Use precomputed the Legendre polynomials found at the 'path'.
     Y_lm_d1_t : array, float, size(2,lmax+1,lmax+1), optional, default = None
         Array with the first derivative
         of Legendre polynomials with respect to colatitude.
@@ -358,9 +399,11 @@ def Displacement_strains(
     Y_lm_d2_tp : array, float, size(2,lmax+1,lmax+1), optional, default = None
         Array with the first derivative
         of Legendre polynomials with respect to colatitude and longitude.
+    y_lm : array, float, size(2,lmax+1,lmax+1), optional, default = None
+        Array of spherical harmonic functions.
     path : string, optional, default = None
-        path where to find the store Legendre polynomials.
-    quiet : int, optional, default = True
+        path where to find the stored Legendre polynomials.
+    quiet : bool, optional, default = True
         If True, suppress printing output.
     """
     if lmax != np.shape(A_lm)[2] - 1:
@@ -374,137 +417,126 @@ def Displacement_strains(
 
     n = 2 * lmax + 2
 
-    if precomp:
-        if Y_lm_d1_p is not None:
-            print("Using loaded precomputed SH derivatives")
-        else:
-            if path is None:
-                raise ValueError(
-                    "Need to speficify the path, here the path is {:s}.".format(
-                        repr(path)
-                    )
-                )
-            Y_lm_d1_t, Y_lm_d1_p, Y_lm_d2_t, Y_lm_d2_p, Y_lm_d2_tp = SH_deriv_store(
-                lmax, path
-            )
-    else:
+    if Y_lm_d1_p is not None:
         if quiet is False:
-            print(
-                "Compute Spherical Harmonic derivatives on the"
-                + " fly, this may be long"
+            print("Using input precomputed SH derivatives")
+    else:
+        if path is None:
+            raise ValueError(
+                "Need to speficify the path, here the path is {:s}.".format(repr(path))
             )
+        (
+            Y_lm_d1_t,
+            Y_lm_d1_p,
+            Y_lm_d2_t,
+            Y_lm_d2_p,
+            Y_lm_d2_tp,
+            y_lm,
+        ) = SH_deriv_store(lmax, path)
 
     # Some constants for the elastic model.
-    Re = R - float(0.5 * Te)
+    Re = R - (0.5 * Te)
     psi = 12.0 * Re ** 2 / Te ** 2
-    D = (E * (Te * Te * Te)) / (float(12.0 * (1.0 - v ** 2)))
+    D = (E * (Te * Te * Te)) / ((12.0 * (1.0 - v ** 2)))
     DpsiTeR = (D * psi) / (Te * R ** 2)
     R_m1 = 1.0 / R
+    n_Rm2 = -(R_m1 ** 2)
+    Te_half = Te / 2.0
 
     # Remove reference radius
     A_lm[0, 0, 0] = 0.0
     w_lm[0, 0, 0] = 0.0
 
     # Allocate arrays.
-    stress_theta = np.zeros((n, 2 * n))
-    stress_phi = np.zeros((n, 2 * n))
-    stress_theta_phi = np.zeros((n, 2 * n))
-    omega = np.zeros((n, 2 * n))
-    kappa_theta = np.zeros((n, 2 * n))
-    kappa_phi = np.zeros((n, 2 * n))
-    tau = np.zeros((n, 2 * n))
-    eps_theta = np.zeros((n, 2 * n))
-    eps_phi = np.zeros((n, 2 * n))
+    shape = (n, 2 * n)
+    omega = np.zeros(shape)
+    kappa_theta = np.zeros(shape)
+    kappa_phi = np.zeros(shape)
+    tau = np.zeros(shape)
+    eps_theta = np.zeros(shape)
+    eps_phi = np.zeros(shape)
 
-    t_i = -1
-    for theta in np.linspace(0, 180, n, endpoint=False) * pi / 180.0:
-        t_i += 1
-        p_i = -1
-        if theta < ((colat_min - 1) * pi / 180.0) or theta > (
-            (colat_max + 1) * pi / 180.0
-        ):
-            # Not in the lat / lon range we investigate.
-            continue
+    deg2rad = pi / 180.0
+    grid_long, grid_lat = np.meshgrid(
+        np.linspace(0, 360, 2 * n, endpoint=False) * deg2rad,
+        np.linspace(0, 180, n, endpoint=False) * deg2rad,
+    )
+    mask = (
+        (grid_lat > (colat_min - 1) * deg2rad)
+        & (grid_lat < (colat_max + 1) * deg2rad)
+        & (grid_long > (lon_min - 1) * deg2rad)
+        & (grid_long < (lon_max + 1) * deg2rad)
+    )
+    sin_g_lat_m = np.sin(grid_lat[mask])
+    csc = np.divide(
+        1.0, sin_g_lat_m, out=np.zeros_like(sin_g_lat_m), where=sin_g_lat_m != 0
+    )
+    csc2 = np.divide(
+        1.0, sin_g_lat_m ** 2, out=np.zeros_like(sin_g_lat_m), where=sin_g_lat_m != 0
+    )
+    cot = np.divide(
+        1.0,
+        np.tan(grid_lat[mask]),
+        out=np.zeros_like(sin_g_lat_m),
+        where=sin_g_lat_m != 0,
+    )
+    cotcsc = csc * cot
 
-        if theta == 0 or theta == pi:
-            csc = 0.0
-            csc2 = 0.0
-            cot = 0.0
-        else:
-            csc = 1.0 / np.sin(theta)
-            csc2 = 1.0 / np.sin(theta) ** 2
-            cot = 1.0 / np.tan(theta)
+    y_lm = y_lm[mask]
+    Y_lm_d2_t = Y_lm_d2_t[mask]
+    Y_lm_d2_p = Y_lm_d2_p[mask]
+    Y_lm_d1_t = Y_lm_d1_t[mask]
+    Y_lm_d1_p = Y_lm_d1_p[mask]
+    Y_lm_d2_tp = Y_lm_d2_tp[mask]
 
-        for phi in np.linspace(0, 360, 2 * n, endpoint=False) * pi / 180.0:
-            p_i += 1
-            if phi < ((lon_min - 1) * pi / 180.0) or phi > ((lon_max + 1) * pi / 180.0):
-                # Not in the lat / lon range we investigate.
-                continue
+    ein_sum = "mijk,ijk->m"
+    ein_sum_mul = "mijk,ijk,m->m"
+    w_deflec_ylm = R_m1 * np.einsum(ein_sum, y_lm, w_lm)
+    eps_theta[mask] = R_m1 * np.einsum(ein_sum, Y_lm_d2_t, A_lm) + w_deflec_ylm
+    eps_phi[mask] = (
+        R_m1
+        * (
+            np.einsum(ein_sum_mul, Y_lm_d2_p, A_lm, csc2)
+            + np.einsum(ein_sum_mul, Y_lm_d1_t, A_lm, cot)
+        )
+        + w_deflec_ylm
+    )
+    omega[mask] = R_m1 * (
+        np.einsum(ein_sum_mul, Y_lm_d2_tp, A_lm, csc)
+        - np.einsum(ein_sum_mul, Y_lm_d1_p, A_lm, cotcsc)
+    )
 
-            if precomp:
-                y_lm = pysh.expand.spharm(lmax, theta, phi, degrees=False)
-                w_lm_ylm = np.sum(w_lm * y_lm)
-                d2Atheta = np.sum(Y_lm_d2_t[t_i, p_i] * A_lm)
-                d2Aphi = np.sum(Y_lm_d2_p[t_i, p_i] * A_lm)
-                d1Atheta = np.sum(Y_lm_d1_t[t_i, p_i] * A_lm)
-                d1Aphi = np.sum(Y_lm_d1_p[t_i, p_i] * A_lm)
-                d2Athetaphi = np.sum(Y_lm_d2_tp[t_i, p_i] * A_lm)
-                d2wtheta = np.sum(Y_lm_d2_t[t_i, p_i] * w_lm)
-                d2wphi = np.sum(Y_lm_d2_p[t_i, p_i] * w_lm)
-                d1wtheta = np.sum(Y_lm_d1_t[t_i, p_i] * w_lm)
-                d1wphi = np.sum(Y_lm_d1_p[t_i, p_i] * w_lm)
-                d2wthetaphi = np.sum(Y_lm_d2_tp[t_i, p_i] * w_lm)
-            else:
-                (
-                    Y_lm_d1_tb,
-                    Y_lm_d1_pb,
-                    Y_lm_d2_tb,
-                    Y_lm_d2_pb,
-                    Y_lm_d2_tpb,
-                    y_lm,
-                ) = SH_deriv(theta, phi, lmax)
-                w_lm_ylm = np.sum(w_lm * y_lm)
-                d2Atheta = np.sum(Y_lm_d2_tb * A_lm)
-                d2Aphi = np.sum(Y_lm_d2_pb * A_lm)
-                d1Atheta = np.sum(Y_lm_d1_tb * A_lm)
-                d1Aphi = np.sum(Y_lm_d1_pb * A_lm)
-                d2Athetaphi = np.sum(Y_lm_d2_tpb * A_lm)
-                d2wtheta = np.sum(Y_lm_d2_tb * w_lm)
-                d2wphi = np.sum(Y_lm_d2_pb * w_lm)
-                d1wtheta = np.sum(Y_lm_d1_tb * w_lm)
-                d1wphi = np.sum(Y_lm_d1_pb * w_lm)
-                d2wthetaphi = np.sum(Y_lm_d2_tpb * w_lm)
+    kappa_theta[mask] = (
+        n_Rm2 * np.einsum(ein_sum, Y_lm_d2_t, w_lm) + (-R_m1) * w_deflec_ylm
+    )
+    kappa_phi[mask] = (
+        n_Rm2
+        * (
+            np.einsum(ein_sum_mul, Y_lm_d2_p, w_lm, csc2)
+            + np.einsum(ein_sum_mul, Y_lm_d1_t, w_lm, cot)
+        )
+        + (-R_m1) * w_deflec_ylm
+    )
+    tau[mask] = n_Rm2 * (
+        np.einsum(ein_sum_mul, Y_lm_d2_tp, w_lm, csc)
+        - np.einsum(ein_sum_mul, Y_lm_d1_p, w_lm, cotcsc)
+    )
 
-            # Sum over theta.
-            eps_theta[t_i, p_i] = R_m1 * (d2Atheta + w_lm_ylm)
-            eps_phi[t_i, p_i] = R_m1 * (d2Aphi * csc2 + d1Atheta * cot + w_lm_ylm)
-            omega[t_i, p_i] = R_m1 * csc * (d2Athetaphi - cot * d1Aphi)
+    stress_theta = (
+        (eps_theta + v * eps_phi + Te_half * (kappa_theta + v * kappa_phi))
+        * DpsiTeR
+        / 1e6
+    )  # MPa
+    stress_phi = (
+        (eps_phi + v * eps_theta + Te_half * (kappa_phi + v * kappa_theta))
+        * DpsiTeR
+        / 1e6
+    )  # MPa
+    stress_theta_phi = (omega + Te_half * tau) * 0.5 * DpsiTeR * (1.0 - v) / 1e6  # MPa
 
-            kappa_theta[t_i, p_i] = -(R_m1 ** 2) * (d2wtheta + w_lm_ylm)
-            kappa_phi[t_i, p_i] = -(R_m1 ** 2) * (
-                d2wphi * csc2 + d1wtheta * cot + w_lm_ylm
-            )
-            tau[t_i, p_i] = -(R_m1 ** 2) * csc * (d2wthetaphi - cot * d1wphi)
-
-            stress_theta[t_i, p_i] = (
-                eps_theta[t_i, p_i]
-                + v * eps_phi[t_i, p_i]
-                + Te / 2.0 * (kappa_theta[t_i, p_i] + v * kappa_phi[t_i, p_i])
-            )
-            stress_phi[t_i, p_i] = (
-                eps_phi[t_i, p_i]
-                + v * eps_theta[t_i, p_i]
-                + Te / 2.0 * (kappa_phi[t_i, p_i] + v * kappa_theta[t_i, p_i])
-            )
-            stress_theta_phi[t_i, p_i] = omega[t_i, p_i] + Te / 2.0 * tau[t_i, p_i]
-
-    stress_theta *= DpsiTeR / 1e6  # MPa
-    stress_phi *= DpsiTeR / 1e6  # MPa
-    stress_theta_phi *= 0.5 * DpsiTeR * (1.0 - v) / 1e6  # MPa
-
-    kappa_theta *= Te / 2.0
-    kappa_phi *= Te / 2.0
-    tau *= Te / 2.0
+    kappa_theta[mask] *= Te_half  # Strain
+    kappa_phi[mask] *= Te_half  # Strain
+    tau[mask] *= Te_half  # Strain
 
     return (
         stress_theta,
@@ -583,31 +615,33 @@ def Plt_tecto_Mars(
     ----------
     path : string
         path for the location of the Knameyer et al (2006) dataset.
-    compression : boolean, optional, default = False
+    compression : bool, optional, default = False
         If True, plot compressive tectonic features.
-    extension : boolean, optional, default = True
+    extension : bool, optional, default = True
         If True, plot extensive tectonic features.
-    ax : object
+    ax : object, optional, default = None
         Matplotlib axis.
-    compression_col : string, default = "k"
+    compression_col : string, optional, default = "k"
         Color of compressive tectonic features.
-    extension_col : string, default = "purple"
+    extension_col : string, optional, default = "purple"
         Color of extensive tectonic features.
-    lw : int, default = 1
+    lw : int, optional, default = 1
         Linewidth for the tectonic features
-    legend_show : boolean, default = True
+    legend_show : bool, optional, default = True
         If True, add a legend to the plot.
-    legend_loc : string, default = "upper left"
+    legend_loc : string, optional, default = "upper left"
         Determine the legend position.
     """
     comp_fault_dat = np.loadtxt("%s/Knapmeyer_2006_compdata.txt" % (path))
     ext_fault_dat = np.loadtxt("%s/Knapmeyer_2006_extedata.txt" % (path))
-    ind_comp_fault = np.isin(comp_fault_dat, np.arange(1, 5143, dtype=float))
+    ind_comp_fault = np.isin(comp_fault_dat, np.arange(1, 5143 + 1, dtype=int))
     ind_comp_fault_2 = np.where(ind_comp_fault)[0]
-    ind_ext_fault = np.isin(ext_fault_dat, np.arange(1, 9676, dtype=float))
+    ind_ext_fault = np.isin(ext_fault_dat, np.arange(1, 9676 + 1, dtype=int))
     ind_ext_fault_2 = np.where(ind_ext_fault)[0]
 
     if ax is None:
+        import matplotlib.pyplot as plt
+
         fig, ax = plt.subplots(1, 1)
 
     faults_inds = [ind_comp_fault_2, ind_ext_fault_2]
@@ -628,11 +662,21 @@ def Plt_tecto_Mars(
 
     for faults, dat, col, label in zip(faults_inds, faults_dats, faults_cols, labels):
         ax.plot(np.nan, np.nan, color=col, lw=lw, label=label)
-        for indx in range(1, int((len(faults) - 1) / 2)):
+        for indx in range(1, len(faults)):
             ind_fault_check = range(faults[indx - 1] + 1, faults[indx])
             fault_dat_lon = dat[ind_fault_check][::2]
             fault_dat_lat = dat[ind_fault_check][1::2]
-            ax.plot((fault_dat_lon + 360) % 360, fault_dat_lat, color=col, lw=lw)
+            split = (
+                np.argwhere((fault_dat_lon[:-1] * fault_dat_lon[1:] < 0)).ravel() + 1
+            )
+            if len(split) > 0:  # Make boundaries periodic by splitting positive
+                # and negative lat lon
+                fault_lon_split = np.split(fault_dat_lon, split)
+                fault_dat_lat = np.split(fault_dat_lat, split)
+                for fault_lon, fault_lat in zip(fault_lon_split, fault_dat_lat):
+                    ax.plot((fault_lon + 360) % 360, fault_lat, color=col, lw=lw)
+            else:
+                ax.plot((fault_dat_lon + 360) % 360, fault_dat_lat, color=col, lw=lw)
 
     if legend_show:
         ax.legend(loc=legend_loc)
